@@ -1,4 +1,4 @@
-# Copyright (c) 2019 by Delphix. All rights reserved.
+# Copyright (c) 2019, 2021 by Delphix. All rights reserved.
 #
 
 # -*- coding: utf-8 -*-
@@ -33,10 +33,12 @@ from dlpx.virtualization.libs.exceptions import (IncorrectArgumentTypeError,
 from dlpx.virtualization.common._common_classes import (RemoteConnection,
                                                         PasswordCredentials,
                                                         KeyPairCredentials)
+from dlpx.virtualization.common.util import response_to_str, to_str
 from google.protobuf import json_format
 from google.protobuf.struct_pb2 import Struct
 
 import logging
+import six
 
 
 __all__ = [
@@ -74,23 +76,24 @@ def _handle_response(response):
 
 
 def _check_exit_code(response, check):
-  """
-  This functions checks the exitcode received in response and throws PluginScriptError
-  if check is True.
+    """
+    This functions checks the exitcode received in response and throws
+    PluginScriptError if check is True.
 
-  Args:
-    response (RunPowerShellResponse or RunBashResponse or RunExpectResponse): Response received by run_bash or
-    run_powershell or run_expect
-    check (bool): if True and non-zero exitcode is received in response, raise PluginScriptError
-  """
-  if (check and response.HasField('return_value')
-          and response.return_value.exit_code != 0):
-    raise PluginScriptError('The script failed with exit code {}.'
-                            ' stdout : {} and '
-                            ' stderr : {}'.format(
-      response.return_value.exit_code,
-      response.return_value.stdout,
-      response.return_value.stderr))
+    Args:
+    response (RunPowerShellResponse or RunBashResponse or RunExpectResponse): Response
+        received by run_bash or run_powershell or run_expect
+    check (bool): if True and non-zero exitcode is received in response, raise
+        PluginScriptError
+    """
+    if (check and response.HasField('return_value')
+            and response.return_value.exit_code != 0):
+        raise PluginScriptError('The script failed with exit code {}.'
+                                ' stdout : {} and '
+                                ' stderr : {}'.format(
+                                      response.return_value.exit_code,
+                                      response.return_value.stdout,
+                                      response.return_value.stderr))
 
 
 def run_bash(remote_connection, command, variables=None, use_login_shell=False,
@@ -128,6 +131,8 @@ def run_bash(remote_connection, command, variables=None, use_login_shell=False,
 
     if variables is None:
         variables = {}
+    command = to_str(command)
+    variables = to_str(variables)
 
     # Validate all the arguments passed in are the right types based on docs.
     if not isinstance(remote_connection, RemoteConnection):
@@ -135,23 +140,23 @@ def run_bash(remote_connection, command, variables=None, use_login_shell=False,
             'remote_connection',
             type(remote_connection),
             RemoteConnection)
-    if not isinstance(command, basestring):
-        raise IncorrectArgumentTypeError('command', type(command), basestring)
+    if not isinstance(command, six.string_types):
+        raise IncorrectArgumentTypeError('command', type(command), six.string_types[0])
     if variables and not isinstance(variables, dict):
         raise IncorrectArgumentTypeError(
             'variables',
             type(variables),
-            {basestring: basestring},
+            {six.string_types[0]: six.string_types[0]},
             False)
-    if (variables and (not all(isinstance(variable, basestring)
+    if (variables and (not all(isinstance(variable, six.string_types)
                                for variable in variables.keys()) or
-                       not all(isinstance(value, basestring)
+                       not all(isinstance(value, six.string_types)
                                for value in variables.values()))):
         raise IncorrectArgumentTypeError(
             'variables',
             {(type(variable), type(value))
              for variable, value in variables.items()},
-            {basestring: basestring},
+            {six.string_types[0]: six.string_types[0]},
             False)
     if use_login_shell and not isinstance(use_login_shell, bool):
         raise IncorrectArgumentTypeError(
@@ -165,6 +170,7 @@ def run_bash(remote_connection, command, variables=None, use_login_shell=False,
         run_bash_request.variables[variable] = value
 
     run_bash_response = internal_libs.run_bash(run_bash_request)
+    response_to_str(run_bash_response)
     _check_exit_code(run_bash_response, check)
     return _handle_response(run_bash_response)
 
@@ -192,46 +198,54 @@ def run_sync(remote_connection, source_directory, rsync_user=None,
 
     from dlpx.virtualization._engine import libs as internal_libs
 
+    source_directory = to_str(source_directory)
+    if rsync_user is not None:
+        rsync_user = to_str(rsync_user)
+    if exclude_paths is not None:
+        exclude_paths = to_str(exclude_paths)
+    if sym_links_to_follow is not None:
+        sym_links_to_follow = to_str(sym_links_to_follow)
+
     # Validate all the arguments passed in are the right types based on docs.
     if not isinstance(remote_connection, RemoteConnection):
         raise IncorrectArgumentTypeError(
             'remote_connection',
             type(remote_connection),
             RemoteConnection)
-    if not isinstance(source_directory, basestring):
+    if not isinstance(source_directory, six.string_types):
         raise IncorrectArgumentTypeError(
-            'source_directory', type(source_directory), basestring)
-    if rsync_user and not isinstance(rsync_user, basestring):
+            'source_directory', type(source_directory), six.string_types[0])
+    if rsync_user and not isinstance(rsync_user, six.string_types):
         raise IncorrectArgumentTypeError(
             'rsync_user',
             type(rsync_user),
-            basestring,
+            six.string_types[0],
             False)
     if exclude_paths and not isinstance(exclude_paths, list):
         raise IncorrectArgumentTypeError(
             'exclude_paths',
             type(exclude_paths),
-            [basestring],
+            [six.string_types[0]],
             False)
     if (exclude_paths and not all(isinstance(
-            path, basestring) for path in exclude_paths)):
+            path, six.string_types) for path in exclude_paths)):
         raise IncorrectArgumentTypeError(
             'exclude_paths',
             [type(path) for path in exclude_paths],
-            [basestring],
+            [six.string_types[0]],
             False)
     if sym_links_to_follow and not isinstance(sym_links_to_follow, list):
         raise IncorrectArgumentTypeError(
             'sym_links_to_follow',
             type(sym_links_to_follow),
-            [basestring],
+            [six.string_types[0]],
             False)
-    if (sym_links_to_follow and not all(isinstance(link, basestring)
+    if (sym_links_to_follow and not all(isinstance(link, six.string_types)
                                         for link in sym_links_to_follow)):
         raise IncorrectArgumentTypeError(
             'sym_links_to_follow',
             [type(link) for link in sym_links_to_follow],
-            [basestring],
+            [six.string_types[0]],
             False)
 
     run_sync_request = libs_pb2.RunSyncRequest()
@@ -245,6 +259,7 @@ def run_sync(remote_connection, source_directory, rsync_user=None,
         run_sync_request.sym_links_to_follow.extend(sym_links_to_follow)
 
     response = internal_libs.run_sync(run_sync_request)
+    response_to_str(response)
     _handle_response(response)
 
 
@@ -282,29 +297,32 @@ def run_powershell(remote_connection, command, variables=None, check=False):
     if variables is None:
         variables = {}
 
+    command = to_str(command)
+    variables = to_str(variables)
+
     # Validate all the arguments passed in are the right types based on docs.
     if not isinstance(remote_connection, RemoteConnection):
         raise IncorrectArgumentTypeError(
             'remote_connection',
             type(remote_connection),
             RemoteConnection)
-    if not isinstance(command, basestring):
-        raise IncorrectArgumentTypeError('command', type(command), basestring)
+    if not isinstance(command, six.string_types):
+        raise IncorrectArgumentTypeError('command', type(command), six.string_types[0])
     if variables and not isinstance(variables, dict):
         raise IncorrectArgumentTypeError(
             'variables',
             type(variables),
-            {basestring: basestring},
+            {six.string_types[0]: six.string_types[0]},
             False)
-    if (variables and (not all(isinstance(variable, basestring)
+    if (variables and (not all(isinstance(variable, six.string_types)
                                for variable in variables.keys()) or
-                       not all(isinstance(value, basestring)
+                       not all(isinstance(value, six.string_types)
                                for value in variables.values()))):
         raise IncorrectArgumentTypeError(
             'variables',
             {(type(variable), type(value))
              for variable, value in variables.items()},
-            {basestring: basestring},
+            {six.string_types[0]: six.string_types[0]},
             False)
 
     run_powershell_request = libs_pb2.RunPowerShellRequest()
@@ -314,6 +332,7 @@ def run_powershell(remote_connection, command, variables=None, check=False):
         run_powershell_request.variables[variable] = value
     run_powershell_response = internal_libs.run_powershell(
         run_powershell_request)
+    response_to_str(run_powershell_response)
     _check_exit_code(run_powershell_response, check)
     return _handle_response(run_powershell_response)
 
@@ -343,33 +362,35 @@ def run_expect(remote_connection, command, variables=None, check=False):
     # scope to allow unit testing of this module.
     #
     from dlpx.virtualization._engine import libs as internal_libs
-
     if variables is None:
         variables = {}
 
+    command = to_str(command)
+    variables = to_str(variables)
+
     # Validate all the arguments passed in are the right types based on docs.
     if not isinstance(remote_connection, RemoteConnection):
         raise IncorrectArgumentTypeError(
             'remote_connection',
             type(remote_connection),
             RemoteConnection)
-    if not isinstance(command, basestring):
-        raise IncorrectArgumentTypeError('command', type(command), basestring)
+    if not isinstance(command, six.string_types):
+        raise IncorrectArgumentTypeError('command', type(command), six.string_types[0])
     if variables and not isinstance(variables, dict):
         raise IncorrectArgumentTypeError(
             'variables',
             type(variables),
-            {basestring: basestring},
+            {six.string_types[0]: six.string_types[0]},
             False)
-    if (variables and (not all(isinstance(variable, basestring)
+    if (variables and (not all(isinstance(variable, six.string_types)
                                for variable in variables.keys()) or
-                       not all(isinstance(value, basestring)
+                       not all(isinstance(value, six.string_types)
                                for value in variables.values()))):
         raise IncorrectArgumentTypeError(
             'variables',
             {(type(variable), type(value))
              for variable, value in variables.items()},
-            {basestring: basestring},
+            {six.string_types[0]: six.string_types[0]},
             False)
 
     run_expect_request = libs_pb2.RunExpectRequest()
@@ -379,6 +400,7 @@ def run_expect(remote_connection, command, variables=None, check=False):
         run_expect_request.variables[variable] = value
 
     run_expect_response = internal_libs.run_expect(run_expect_request)
+    response_to_str(run_expect_response)
     _check_exit_code(run_expect_response, check)
     return _handle_response(run_expect_response)
 
@@ -400,6 +422,7 @@ def _log_request(message, log_level):
     """
     from dlpx.virtualization._engine import libs as internal_libs
 
+    message = to_str(message)
     log_request = libs_pb2.LogRequest()
     log_request.message = message
 
@@ -415,23 +438,27 @@ def _log_request(message, log_level):
         log_request.level = libs_pb2.LogRequest.ERROR
 
     response = internal_libs.log(log_request)
+    response_to_str(response)
     _handle_response(response)
 
 
 def retrieve_credentials(credentials_supplier):
-    """This is an internal wrapper around the Virtualization library's credentials retrieval API.
-    Given a supplier provided by Virtualization, retrieves the credentials from that supplier.
+    """
+    This is an internal wrapper around the Virtualization library's credentials
+    retrieval API. Given a supplier provided by Virtualization, retrieves the
+    credentials from that supplier.
 
     Args:
         credentials_supplier (dict): Properties that make up a supplier of credentials.
     Return:
-        Subclass of Credentials retrieved from supplier. Either a PasswordCredentials or a KeyPairCredentials
-        from dlpx.virtualization.common._common_classes.
+        Subclass of Credentials retrieved from supplier. Either a PasswordCredentials
+        or a KeyPairCredentials from dlpx.virtualization.common._common_classes.
     """
     from dlpx.virtualization._engine import libs as internal_libs
 
     if not isinstance(credentials_supplier, dict):
-        raise IncorrectArgumentTypeError('credentials_supplier', type(credentials_supplier), dict)
+        raise IncorrectArgumentTypeError(
+            'credentials_supplier', type(credentials_supplier), dict)
 
     credentials_request = libs_pb2.CredentialsRequest()
     credentials_struct = Struct()
@@ -439,10 +466,11 @@ def retrieve_credentials(credentials_supplier):
     credentials_request.credentials_supplier.CopyFrom(credentials_struct)
 
     response = internal_libs.retrieve_credentials(credentials_request)
-
+    response_to_str(response)
     credentials_result = _handle_response(response)
     if credentials_result.password != "":
-        return PasswordCredentials(credentials_result.username, credentials_result.password)
+        return PasswordCredentials(
+            credentials_result.username, credentials_result.password)
     return KeyPairCredentials(
         credentials_result.username,
         credentials_result.key_pair.private_key,
@@ -450,22 +478,26 @@ def retrieve_credentials(credentials_supplier):
 
 
 def upgrade_password(password, username=None):
-    """This is an internal wrapper around Virtualization's credentials-supplier conversion  API.
-    It is intended for use during plugin upgrade when a plugin needs to transform a password
-    value into a more generic credentials supplier object.
+    """
+    This is an internal wrapper around Virtualization's credentials-supplier conversion
+    API. It is intended for use during plugin upgrade when a plugin needs to transform
+    a password value into a more generic credentials supplier object.
 
     Args:
-        password (basestring): Plain password string.
-        username (basestring, defaults to None): User name contained in the password credential supplier to return.
+        password (str): Plain password string.
+        username (str, defaults to None): User name contained in the password
+            credential supplier to return.
     Return:
         Credentials supplier (dict) that supplies the given password and username.
     """
     from dlpx.virtualization._engine import libs as internal_libs
 
-    if not isinstance(password, basestring):
-        raise IncorrectArgumentTypeError('password', type(password), basestring)
-    if username and not isinstance(username, basestring):
-        raise IncorrectArgumentTypeError('username', type(username), basestring, required=False)
+    if not isinstance(password, six.string_types):
+        raise IncorrectArgumentTypeError(
+            'password', type(password), six.string_types[0])
+    if username and not isinstance(username, six.string_types):
+        raise IncorrectArgumentTypeError(
+            'username', type(username), six.string_types[0], required=False)
 
     upgrade_password_request = libs_pb2.UpgradePasswordRequest()
     upgrade_password_request.password = password
@@ -473,6 +505,6 @@ def upgrade_password(password, username=None):
         upgrade_password_request.username = username
 
     response = internal_libs.upgrade_password(upgrade_password_request)
-
+    response_to_str(response)
     upgrade_password_result = _handle_response(response)
     return json_format.MessageToDict(upgrade_password_result.credentials_supplier)
